@@ -11,6 +11,10 @@
 #include "xz_stream.h"
 #include "crc32.h"
 
+static const unsigned char HEADER_MAGIC[HEADER_MAGIC_SIZE] = {
+	0xFD, '7', 'z', 'X', 'Z', 0x00
+};
+
 #define xz_crc32 libxmp_crc32_A
 
 #ifdef XZ_USE_CRC64
