@@ -25,6 +25,7 @@
 #include "extras.h"
 #include "med_extras.h"
 #include "hmn_extras.h"
+#include "far_extras.h"
 
 /*
  * Module extras
@@ -38,6 +39,8 @@ void libxmp_release_module_extras(struct context_data *ctx)
 		libxmp_med_release_module_extras(m);
 	else if (HAS_HMN_MODULE_EXTRAS(*m))
 		libxmp_hmn_release_module_extras(m);
+	else if (HAS_FAR_MODULE_EXTRAS(*m))
+		libxmp_far_release_module_extras(m);
 }
 
 /*
@@ -54,6 +57,9 @@ int libxmp_new_channel_extras(struct context_data *ctx, struct channel_data *xc)
 	} else if (HAS_HMN_MODULE_EXTRAS(*m)) {
 		if (libxmp_hmn_new_channel_extras(xc) < 0)
 			return -1;
+	} else if (HAS_FAR_MODULE_EXTRAS(*m)) {
+		if (libxmp_far_new_channel_extras(xc) < 0)
+			return -1;
 	}
 
 	return 0;
@@ -67,6 +73,8 @@ void libxmp_release_channel_extras(struct context_data *ctx, struct channel_data
 		libxmp_med_release_channel_extras(xc);
 	else if (HAS_HMN_CHANNEL_EXTRAS(*m))
 		libxmp_hmn_release_channel_extras(xc);
+	else if (HAS_FAR_CHANNEL_EXTRAS(*m))
+		libxmp_far_release_channel_extras(xc);
 }
 
 void libxmp_reset_channel_extras(struct context_data *ctx, struct channel_data *xc)
@@ -77,6 +85,8 @@ void libxmp_reset_channel_extras(struct context_data *ctx, struct channel_data *
 		libxmp_med_reset_channel_extras(xc);
 	else if (HAS_HMN_CHANNEL_EXTRAS(*m))
 		libxmp_hmn_reset_channel_extras(xc);
+	else if (HAS_FAR_CHANNEL_EXTRAS(*m))
+		libxmp_far_reset_channel_extras(xc);
 }
 
 /*
@@ -87,12 +97,15 @@ void libxmp_play_extras(struct context_data *ctx, struct channel_data *xc, int c
 {
 	struct module_data *m = &ctx->m;
 
+	if (HAS_FAR_CHANNEL_EXTRAS(*xc))
+		libxmp_far_play_extras(ctx, xc, chn);
+
 	if (xc->ins >= m->mod.ins)	/* SFX instruments have no extras */
 		return;
 
-        if (HAS_MED_INSTRUMENT_EXTRAS(m->mod.xxi[xc->ins]))
+	if (HAS_MED_INSTRUMENT_EXTRAS(m->mod.xxi[xc->ins]))
 		libxmp_med_play_extras(ctx, xc, chn);
-        else if (HAS_HMN_INSTRUMENT_EXTRAS(m->mod.xxi[xc->ins]))
+	else if (HAS_HMN_INSTRUMENT_EXTRAS(m->mod.xxi[xc->ins]))
 		libxmp_hmn_play_extras(ctx, xc, chn);
 }
 
@@ -103,7 +116,7 @@ int libxmp_extras_get_volume(struct context_data *ctx, struct channel_data *xc)
 
 	if (xc->ins >= m->mod.ins)
 		vol = xc->volume;
-        else if (HAS_MED_INSTRUMENT_EXTRAS(m->mod.xxi[xc->ins]))
+	else if (HAS_MED_INSTRUMENT_EXTRAS(m->mod.xxi[xc->ins]))
 		vol = MED_CHANNEL_EXTRAS(*xc)->volume * xc->volume / 64;
 	else if (HAS_HMN_INSTRUMENT_EXTRAS(m->mod.xxi[xc->ins]))
 		vol = HMN_CHANNEL_EXTRAS(*xc)->volume * xc->volume / 64;
@@ -145,4 +158,6 @@ void libxmp_extras_process_fx(struct context_data *ctx, struct channel_data *xc,
 		libxmp_med_extras_process_fx(ctx, xc, chn, note, fxt, fxp, fnum);
 	else if (HAS_HMN_CHANNEL_EXTRAS(*xc))
 		libxmp_hmn_extras_process_fx(ctx, xc, chn, note, fxt, fxp, fnum);
+	else if (HAS_FAR_CHANNEL_EXTRAS(*xc))
+		libxmp_far_extras_process_fx(ctx, xc, chn, note, fxt, fxp, fnum);
 }
