@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2022 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2024 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -390,7 +390,7 @@ HIO_HANDLE *hio_open(const char *path, const char *mode)
 	return NULL;
 }
 
-HIO_HANDLE *hio_open_mem(const void *ptr, long size, int free_after_use)
+HIO_HANDLE *hio_open_const_mem(const void *ptr, long size)
 {
 	HIO_HANDLE *h;
 
@@ -400,7 +400,7 @@ HIO_HANDLE *hio_open_mem(const void *ptr, long size, int free_after_use)
 		return NULL;
 
 	h->type = HIO_HANDLE_TYPE_MEMORY;
-	h->handle.mem = mopen(ptr, size, free_after_use);
+	h->handle.mem = mcopen(ptr, size);
 	h->size = size;
 
 	if (!h->handle.mem) {
@@ -486,7 +486,7 @@ static int hio_close_internal(HIO_HANDLE *h)
 }
 
 /* hio_close + hio_open_mem. Reuses the same HIO_HANDLE. */
-int hio_reopen_mem(const void *ptr, long size, int free_after_use, HIO_HANDLE *h)
+int hio_reopen_mem(void *ptr, long size, int free_after_use, HIO_HANDLE *h)
 {
 	MFILE *m;
 	int ret;
@@ -499,7 +499,7 @@ int hio_reopen_mem(const void *ptr, long size, int free_after_use, HIO_HANDLE *h
 
 	ret = hio_close_internal(h);
 	if (ret < 0) {
-		m->free_after_use = 0;
+		m->ptr_free = NULL;
 		mclose(m);
 		return ret;
 	}
@@ -542,3 +542,19 @@ long hio_size(HIO_HANDLE *h)
 {
 	return h->size;
 }
+
+/* Returns a pointer to the underlying continuous memory buffer the entire
+ * contents of HIO_HANDLE `h` are stored at if applicable, otherwise NULL.
+ * Do not reallocate this pointer or modify its underlying data!
+ */
+const unsigned char *hio_get_underlying_memory(HIO_HANDLE *h)
+{
+	switch (HIO_HANDLE_TYPE(h)) {
+	case HIO_HANDLE_TYPE_FILE:
+	case HIO_HANDLE_TYPE_CBFILE:
+		return NULL;
+	case HIO_HANDLE_TYPE_MEMORY:
+		return h->handle.mem->start;
+	}
+	return NULL;
+}
