@@ -38,13 +38,14 @@ long	hio_tell	(HIO_HANDLE *);
 int	hio_eof		(HIO_HANDLE *);
 int	hio_error	(HIO_HANDLE *);
 HIO_HANDLE *hio_open	(const char *, const char *);
-HIO_HANDLE *hio_open_mem  (const void *, long, int);
+HIO_HANDLE *hio_open_const_mem  (const void *, long);
 HIO_HANDLE *hio_open_file (FILE *);
 HIO_HANDLE *hio_open_file2 (FILE *);/* allows fclose()ing the file by libxmp */
 HIO_HANDLE *hio_open_callbacks (void *, struct xmp_callbacks);
-int	hio_reopen_mem	(const void *, long, int, HIO_HANDLE *);
+int	hio_reopen_mem	(void *, long, int, HIO_HANDLE *);
 int	hio_reopen_file	(FILE *, int, HIO_HANDLE *);
 int	hio_close	(HIO_HANDLE *);
 long	hio_size	(HIO_HANDLE *);
+const unsigned char *hio_get_underlying_memory(HIO_HANDLE *);
 
 #endif
