@@ -22,8 +22,8 @@
  */
 
 #include "loader.h"
-#include "effects.h"
-#include "period.h"
+#include "../effects.h"
+#include "../period.h"
 
 #define AMOS_BANK 0x416d426b
 #define AMOS_MUSIC_TYPE 0x0003
@@ -36,7 +36,7 @@
 static int abk_test (HIO_HANDLE *, char *, const int);
 static int abk_load (struct module_data *, HIO_HANDLE *, const int);
 
-struct format_loader libxmp_loader_abk =
+const struct format_loader libxmp_loader_abk =
 {
     "AMOS Music Bank",
     abk_test,
@@ -172,7 +172,7 @@ static int read_abk_song(HIO_HANDLE *f, struct abk_song *song, uint32 songs_sect
 }
 
 /**
- * @brief reads an ABK pattern into a xmp_event structure. 
+ * @brief reads an ABK pattern into a xmp_event structure.
  * @param f stream to read data from.
  * @param events events object to populate.
  * @param pattern_offset_abs the absolute file offset to the start of the patter to read.
@@ -364,7 +364,7 @@ static int read_abk_pattern(HIO_HANDLE *f, struct xmp_event *events, uint32 patt
             break;
         }
     }
-    if (position < 64) {
+    if (position >= 1 && position < 64) {
 	events[position - 1].f2t = FX_BREAK;
     }
 
@@ -387,20 +387,20 @@ static struct abk_instrument* read_abk_insts(HIO_HANDLE *f, uint32 inst_section_
     for (i = 0; i < count; i++)
     {
         uint32 sampleLength;
-    
+
         inst[i].sample_offset = hio_read32b(f);
         inst[i].repeat_offset = hio_read32b(f);
         inst[i].sample_length = hio_read16b(f);
         inst[i].repeat_end = hio_read16b(f);
         inst[i].sample_volume = hio_read16b(f);
         sampleLength = hio_read16b(f);
-    
+
         /* detect a potential bug where the sample length is not specified (and we might already know the length) */
         if (sampleLength > 4)
         {
             inst[i].sample_length = sampleLength;
         }
-    
+
         if (hio_read(inst[i].sample_name, 1, 16, f) != 16) {
             free(inst);
             return NULL;
@@ -412,7 +412,9 @@ static struct abk_instrument* read_abk_insts(HIO_HANDLE *f, uint32 inst_section_
 
 static int abk_test(HIO_HANDLE *f, char *t, const int start)
 {
+    struct abk_song song;
     char music[8];
+    uint32 song_section_offset;
 
     if (hio_read32b(f) != AMOS_BANK)
     {
@@ -427,13 +429,23 @@ static int abk_test(HIO_HANDLE *f, char *t, const int start)
     /* skip over length and chip/fastmem.*/
     hio_seek(f, 6, SEEK_CUR);
 
-    hio_read(music, 1, 8, f);	/* get the "Music   " */
+    if (hio_read(music, 1, 8, f) != 8)	/* get the "Music   " */
+        return -1;
 
     if (memcmp(music, "Music   ", 8))
     {
         return -1;
     }
 
+    /* Attempt to read title. */
+    hio_read32b(f); /* instruments_offset */
+    song_section_offset = hio_read32b(f);
+
+    if (t != NULL && read_abk_song(f, &song, AMOS_MAIN_HEADER + song_section_offset) == 0)
+    {
+        libxmp_copy_adjust(t, (uint8 *)song.song_name, AMOS_STRING_LEN);
+    }
+
     return 0;
 }
 
@@ -595,7 +607,7 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
         {
             pattern = hio_read16b(f);
             if (read_abk_pattern(f,  mod->xxt[(i*mod->chn)+k]->event, AMOS_MAIN_HEADER + main_header.patterns_offset + pattern) < 0) {
-    		free(playlist.pattern);
+		free(playlist.pattern);
 		return -1;
 	    }
         }
@@ -608,7 +620,7 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
     	free(playlist.pattern);
 	return -1;
     }
-	
+
     mod->len = playlist.length;
 
     /* now push all the patterns into the module and set the length */
