@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2024 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -54,11 +54,11 @@ static int stim_test(HIO_HANDLE *f, char *t, const int start)
 }
 
 struct stim_instrument {
-	uint16 size;		/* Lenght of the sample (/2) */
+	uint16 size;		/* Length of the sample (/2) */
 	uint8 finetune;		/* Finetune (as ptk) */
 	uint8 volume;		/* Volume (as ptk) */
 	uint16 loop_start;	/* Loop start (/2) */
-	uint16 loop_size;	/* Loop lenght (/2) */
+	uint16 loop_size;	/* Loop length (/2) */
 };
 
 struct stim_header {
