@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2023 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2024 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,19 +32,14 @@
 #endif
 
 
-#define FLAG_16_BITS	0x01
-#define FLAG_STEREO	0x02
-#define FLAG_FILTER	0x04
-#define FLAG_ACTIVE	0x10
-/* #define FLAG_SYNTH	0x20 */
-#define FIDX_FLAGMASK	(FLAG_16_BITS | FLAG_STEREO | FLAG_FILTER)
-
 #define DOWNMIX_SHIFT	 12
 #define LIM8_HI		 127
 #define LIM8_LO		-128
 #define LIM16_HI	 32767
 #define LIM16_LO	-32768
 
+#define ANTICLICK_FPSHIFT	24
+
 struct loop_data
 {
 #define LOOP_PROLOGUE 1
@@ -55,118 +50,106 @@ struct loop_data
 	int first_loop;
 	int _16bit;
 	int active;
-	uint32 prologue[LOOP_PROLOGUE];
-	uint32 epilogue[LOOP_EPILOGUE];
+	int prologue_num;
+	int epilogue_num;
+	uint8 prologue[LOOP_PROLOGUE * 2 /* 16-bit */ * 2 /* stereo */];
+	uint8 epilogue[LOOP_EPILOGUE * 2 /* 16-bit */ * 2 /* stereo */];
 };
 
-#define MIX_FN(x) void libxmp_mix_##x(struct mixer_voice *, int32 *, int, int, int, int, int, int, int)
-
-#define ANTICLICK_FPSHIFT	24
-
-MIX_FN(mono_8bit_nearest);
-MIX_FN(mono_8bit_linear);
-MIX_FN(mono_16bit_nearest);
-MIX_FN(mono_16bit_linear);
-MIX_FN(stereo_8bit_nearest);
-MIX_FN(stereo_8bit_linear);
-MIX_FN(stereo_16bit_nearest);
-MIX_FN(stereo_16bit_linear);
-MIX_FN(mono_8bit_spline);
-MIX_FN(mono_16bit_spline);
-MIX_FN(stereo_8bit_spline);
-MIX_FN(stereo_16bit_spline);
-
-#ifndef LIBXMP_CORE_DISABLE_IT
-MIX_FN(mono_8bit_linear_filter);
-MIX_FN(mono_16bit_linear_filter);
-MIX_FN(stereo_8bit_linear_filter);
-MIX_FN(stereo_16bit_linear_filter);
-MIX_FN(mono_8bit_spline_filter);
-MIX_FN(mono_16bit_spline_filter);
-MIX_FN(stereo_8bit_spline_filter);
-MIX_FN(stereo_16bit_spline_filter);
-#endif
-
-#ifdef LIBXMP_PAULA_SIMULATOR
-MIX_FN(mono_a500);
-MIX_FN(mono_a500_filter);
-MIX_FN(stereo_a500);
-MIX_FN(stereo_a500_filter);
-#endif
-
 /* Mixers array index:
  *
  * bit 0: 0=8 bit sample, 1=16 bit sample
- * bit 1: 0=mono output, 1=stereo output
- * bit 2: 0=unfiltered, 1=filtered
+ * bit 1: 0=mono sample, 1=stereo sample
+ * bit 2: 0=mono output, 1=stereo output
+ * bit 3: 0=unfiltered, 1=filtered
  */
 
-typedef void (*MIX_FP) (struct mixer_voice *, int32 *, int, int, int, int, int, int, int);
+#define FLAG_16_BITS	0x01
+#define FLAG_STEREO	0x02
+#define FLAG_STEREOOUT	0x04
+#define FLAG_FILTER	0x08
+#define FLAG_ACTIVE	0x10
+/* #define FLAG_SYNTH	0x20 */
+#define FIDX_FLAGMASK	(FLAG_16_BITS | FLAG_STEREO | FLAG_STEREOOUT | FLAG_FILTER)
 
-static MIX_FP nearest_mixers[] = {
-	libxmp_mix_mono_8bit_nearest,
-	libxmp_mix_mono_16bit_nearest,
-	libxmp_mix_stereo_8bit_nearest,
-	libxmp_mix_stereo_16bit_nearest,
+#define MIX_FN(x) void libxmp_mix_##x(struct mixer_voice * LIBXMP_RESTRICT, \
+	int32 * LIBXMP_RESTRICT, int, int, int, int, int, int, int)
+
+#define DECLARE_MIX_FUNCTIONS(type) \
+	MIX_FN(monoout_mono_8bit_ ## type); \
+	MIX_FN(monoout_mono_16bit_ ## type); \
+	MIX_FN(monoout_stereo_8bit_ ## type); \
+	MIX_FN(monoout_stereo_16bit_ ## type); \
+	MIX_FN(stereoout_mono_8bit_ ## type); \
+	MIX_FN(stereoout_mono_16bit_ ## type); \
+	MIX_FN(stereoout_stereo_8bit_ ## type); \
+	MIX_FN(stereoout_stereo_16bit_ ## type)
+
+#define LIST_MIX_FUNCTIONS(type) \
+	libxmp_mix_monoout_mono_8bit_ ## type, \
+	libxmp_mix_monoout_mono_16bit_ ## type, \
+	libxmp_mix_monoout_stereo_8bit_ ## type, \
+	libxmp_mix_monoout_stereo_16bit_ ## type, \
+	libxmp_mix_stereoout_mono_8bit_ ## type, \
+	libxmp_mix_stereoout_mono_16bit_ ## type, \
+	libxmp_mix_stereoout_stereo_8bit_ ## type, \
+	libxmp_mix_stereoout_stereo_16bit_ ## type
+
+DECLARE_MIX_FUNCTIONS(nearest);
+DECLARE_MIX_FUNCTIONS(linear);
+DECLARE_MIX_FUNCTIONS(spline);
 
 #ifndef LIBXMP_CORE_DISABLE_IT
-	libxmp_mix_mono_8bit_nearest,
-	libxmp_mix_mono_16bit_nearest,
-	libxmp_mix_stereo_8bit_nearest,
-	libxmp_mix_stereo_16bit_nearest,
+DECLARE_MIX_FUNCTIONS(linear_filter);
+DECLARE_MIX_FUNCTIONS(spline_filter);
+#endif
+
+#ifdef LIBXMP_PAULA_SIMULATOR
+MIX_FN(monoout_mono_a500);
+MIX_FN(monoout_mono_a500_filter);
+MIX_FN(stereoout_mono_a500);
+MIX_FN(stereoout_mono_a500_filter);
+#endif
+
+typedef void (*MIX_FP) (struct mixer_voice* LIBXMP_RESTRICT, int32* LIBXMP_RESTRICT, int, int, int, int, int, int, int);
+
+static const MIX_FP nearest_mixers[] = {
+	LIST_MIX_FUNCTIONS(nearest),
+
+#ifndef LIBXMP_CORE_DISABLE_IT
+	LIST_MIX_FUNCTIONS(nearest)
 #endif
 };
 
-static MIX_FP linear_mixers[] = {
-	libxmp_mix_mono_8bit_linear,
-	libxmp_mix_mono_16bit_linear,
-	libxmp_mix_stereo_8bit_linear,
-	libxmp_mix_stereo_16bit_linear,
+static const MIX_FP linear_mixers[] = {
+	LIST_MIX_FUNCTIONS(linear),
 
 #ifndef LIBXMP_CORE_DISABLE_IT
-	libxmp_mix_mono_8bit_linear_filter,
-	libxmp_mix_mono_16bit_linear_filter,
-	libxmp_mix_stereo_8bit_linear_filter,
-	libxmp_mix_stereo_16bit_linear_filter
+	LIST_MIX_FUNCTIONS(linear_filter)
 #endif
 };
 
-static MIX_FP spline_mixers[] = {
-	libxmp_mix_mono_8bit_spline,
-	libxmp_mix_mono_16bit_spline,
-	libxmp_mix_stereo_8bit_spline,
-	libxmp_mix_stereo_16bit_spline,
+static const MIX_FP spline_mixers[] = {
+	LIST_MIX_FUNCTIONS(spline),
 
 #ifndef LIBXMP_CORE_DISABLE_IT
-	libxmp_mix_mono_8bit_spline_filter,
-	libxmp_mix_mono_16bit_spline_filter,
-	libxmp_mix_stereo_8bit_spline_filter,
-	libxmp_mix_stereo_16bit_spline_filter
+	LIST_MIX_FUNCTIONS(spline_filter)
 #endif
 };
 
 #ifdef LIBXMP_PAULA_SIMULATOR
-static MIX_FP a500_mixers[] = {
-	libxmp_mix_mono_a500,
-	NULL,
-	libxmp_mix_stereo_a500,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL
+#define LIST_MIX_FUNCTIONS_PAULA(type) \
+	libxmp_mix_monoout_mono_ ## type, NULL, NULL, NULL, \
+	libxmp_mix_stereoout_mono_ ## type, NULL, NULL, NULL, \
+	NULL, NULL, NULL, NULL, \
+	NULL, NULL, NULL, NULL
+
+static const MIX_FP a500_mixers[] = {
+	LIST_MIX_FUNCTIONS_PAULA(a500)
 };
 
-
-static MIX_FP a500led_mixers[] = {
-	libxmp_mix_mono_a500_filter,
-	NULL,
-	libxmp_mix_stereo_a500_filter,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL
+static const MIX_FP a500led_mixers[] = {
+	LIST_MIX_FUNCTIONS_PAULA(a500_filter)
 };
 #endif
 
@@ -230,8 +213,8 @@ static void do_anticlick(struct context_data *ctx, int voc, int32 *buf, int coun
 	int stepmul, stepval;
 	uint32 stepmul_sq;
 
-	smp_r = vi->sright;
 	smp_l = vi->sleft;
+	smp_r = vi->sright;
 	vi->sright = vi->sleft = 0;
 
 	if (smp_l == 0 && smp_r == 0) {
@@ -257,8 +240,8 @@ static void do_anticlick(struct context_data *ctx, int voc, int32 *buf, int coun
 			/* Truncate to 16-bits of precision so the product is 32-bits. */
 			stepmul_sq = stepmul >> (ANTICLICK_FPSHIFT - 16);
 			stepmul_sq *= stepmul_sq;
-			*buf++ += (stepmul_sq * (int64)smp_r) >> 32;
 			*buf++ += (stepmul_sq * (int64)smp_l) >> 32;
+			*buf++ += (stepmul_sq * (int64)smp_r) >> 32;
 		}
 	} else {
 		while ((stepmul -= stepval) > 0) {
@@ -293,10 +276,14 @@ static void set_sample_end(struct context_data *ctx, int voc, int end)
 }
 
 /* Back up sample data before and after loop and replace it for interpolation.
+ * TODO: if higher order interpolation than spline is added, the copy needs to
+ *       properly wrap around the loop data (modulo) for correct small loops.
  * TODO: use an overlap buffer like OpenMPT? This is easier, but a little dirty. */
 static void init_sample_wraparound(struct mixer_data *s, struct loop_data *ld,
 				   struct mixer_voice *vi, struct xmp_sample *xxs)
 {
+	int prologue_num = LOOP_PROLOGUE;
+	int epilogue_num = LOOP_EPILOGUE;
 	int bidir;
 	int i;
 
@@ -312,36 +299,48 @@ static void init_sample_wraparound(struct mixer_data *s, struct loop_data *ld,
 	ld->_16bit = (xxs->flg & XMP_SAMPLE_16BIT);
 	ld->active = 1;
 
+	/* Stereo */
+	if (xxs->flg & XMP_SAMPLE_STEREO) {
+		ld->start <<= 1;
+		ld->end <<= 1;
+		prologue_num <<= 1;
+		epilogue_num <<= 1;
+	}
+	ld->prologue_num = prologue_num;
+	ld->epilogue_num = epilogue_num;
+
 	bidir = vi->flags & VOICE_BIDIR;
 
 	if (ld->_16bit) {
-		uint16 *start = (uint16 *)vi->sptr + vi->start;
-		uint16 *end = (uint16 *)vi->sptr + vi->end;
+		uint16 *start = (uint16 *)ld->sptr + ld->start;
+		uint16 *end = (uint16 *)ld->sptr + ld->end;
+
+		memcpy(ld->prologue, start - prologue_num, prologue_num * 2);
+		memcpy(ld->epilogue, end, epilogue_num * 2);
 
 		if (!ld->first_loop) {
-			for (i = 0; i < LOOP_PROLOGUE; i++) {
-				int j = i - LOOP_PROLOGUE;
-				ld->prologue[i] = start[j];
+			for (i = 0; i < prologue_num; i++) {
+				int j = i - prologue_num;
 				start[j] = bidir ? start[-1 - j] : end[j];
 			}
 		}
-		for (i = 0; i < LOOP_EPILOGUE; i++) {
-			ld->epilogue[i] = end[i];
+		for (i = 0; i < epilogue_num; i++) {
 			end[i] = bidir ? end[-1 - i] : start[i];
 		}
 	} else {
-		uint8 *start = (uint8 *)vi->sptr + vi->start;
-		uint8 *end = (uint8 *)vi->sptr + vi->end;
+		uint8 *start = (uint8 *)ld->sptr + ld->start;
+		uint8 *end = (uint8 *)ld->sptr + ld->end;
+
+		memcpy(ld->prologue, start - prologue_num, prologue_num);
+		memcpy(ld->epilogue, end, epilogue_num);
 
 		if (!ld->first_loop) {
-			for (i = 0; i < LOOP_PROLOGUE; i++) {
-				int j = i - LOOP_PROLOGUE;
-				ld->prologue[i] = start[j];
+			for (i = 0; i < prologue_num; i++) {
+				int j = i - prologue_num;
 				start[j] = bidir ? start[-1 - j] : end[j];
 			}
 		}
-		for (i = 0; i < LOOP_EPILOGUE; i++) {
-			ld->epilogue[i] = end[i];
+		for (i = 0; i < epilogue_num; i++) {
 			end[i] = bidir ? end[-1 - i] : start[i];
 		}
 	}
@@ -350,7 +349,8 @@ static void init_sample_wraparound(struct mixer_data *s, struct loop_data *ld,
 /* Restore old sample data from before and after loop. */
 static void reset_sample_wraparound(struct loop_data *ld)
 {
-	int i;
+	int prologue_num = ld->prologue_num;
+	int epilogue_num = ld->epilogue_num;
 
 	if (!ld->active)
 		return;
@@ -359,22 +359,14 @@ static void reset_sample_wraparound(struct loop_data *ld)
 		uint16 *start = (uint16 *)ld->sptr + ld->start;
 		uint16 *end = (uint16 *)ld->sptr + ld->end;
 
-		if (!ld->first_loop) {
-			for (i = 0; i < LOOP_PROLOGUE; i++)
-				start[i - LOOP_PROLOGUE] = ld->prologue[i];
-		}
-		for (i = 0; i < LOOP_EPILOGUE; i++)
-			end[i] = ld->epilogue[i];
+		memcpy(start - prologue_num, ld->prologue, prologue_num * 2);
+		memcpy(end, ld->epilogue, epilogue_num * 2);
 	} else {
 		uint8 *start = (uint8 *)ld->sptr + ld->start;
 		uint8 *end = (uint8 *)ld->sptr + ld->end;
 
-		if (!ld->first_loop) {
-			for (i = 0; i < LOOP_PROLOGUE; i++)
-				start[i - LOOP_PROLOGUE] = ld->prologue[i];
-		}
-		for (i = 0; i < LOOP_EPILOGUE; i++)
-			end[i] = ld->epilogue[i];
+		memcpy(start - prologue_num, ld->prologue, prologue_num);
+		memcpy(end, ld->epilogue, epilogue_num);
 	}
 }
 
@@ -450,9 +442,66 @@ static int loop_reposition(struct context_data *ctx, struct mixer_voice *vi,
 			vi->pos = vi->start * 2 - vi->pos;
 		}
 	}
+	/* Safety check: pos should not be excessively past the sample end.
+	 * This only seems to happen with very low sample rates. */
+	if (vi->pos > xxs->len + 1) {
+		vi->pos = xxs->len + 1;
+	}
 	return loop_changed;
 }
 
+static void hotswap_sample(struct context_data *ctx, struct mixer_voice *vi,
+ int voc, int smp)
+{
+	int vol = vi->vol;
+	int pan = vi->pan;
+	libxmp_mixer_setpatch(ctx, voc, smp, 0);
+	vi->flags |= SAMPLE_LOOP;
+	vi->vol = vol;
+	vi->pan = pan;
+}
+
+static void get_current_sample(struct context_data *ctx, struct mixer_voice *vi,
+ struct xmp_sample **xxs, struct extra_sample_data **xtra, int *c5spd)
+{
+	struct module_data *m = &ctx->m;
+	struct xmp_module *mod = &m->mod;
+
+	if (vi->smp < mod->smp) {
+		*xxs = &mod->xxs[vi->smp];
+		*xtra = &m->xtra[vi->smp];
+		*c5spd = m->xtra[vi->smp].c5spd;
+	} else {
+		*xxs = &ctx->smix.xxs[vi->smp - mod->smp];
+		*xtra = NULL;
+		*c5spd = m->c4rate;
+	}
+	adjust_voice_end(ctx, vi, *xxs, *xtra);
+}
+
+/* Calculate the required number of sample frames to render a tick.
+ * Returns -1 if any of the parameters are invalid. */
+int libxmp_mixer_get_ticksize(int freq, double time_factor, double rrate, int bpm)
+{
+	double calc;
+	int ticksize;
+
+	if (freq <= 0 || bpm <= 0 || time_factor <= 0.0 || rrate <= 0.0) {
+		return -1;
+	}
+
+	calc = freq * time_factor * rrate / bpm / 1000;
+	if (calc > INT_MAX || calc != calc /* NaN */) {
+		return -1;
+	}
+
+	ticksize = (int)calc;
+
+	if (ticksize < (1 << ANTICLICK_SHIFT))
+		ticksize = 1 << ANTICLICK_SHIFT;
+
+	return ticksize;
+}
 
 /* Prepare the mixer for the next tick */
 void libxmp_mixer_prepare(struct context_data *ctx)
@@ -462,10 +511,12 @@ void libxmp_mixer_prepare(struct context_data *ctx)
 	struct mixer_data *s = &ctx->s;
 	int bytelen;
 
-	s->ticksize = s->freq * m->time_factor * m->rrate / p->bpm / 1000;
+	s->ticksize = libxmp_mixer_get_ticksize(s->freq, m->time_factor, m->rrate, p->bpm);
 
-	if (s->ticksize < (1 << ANTICLICK_SHIFT))
-		s->ticksize = 1 << ANTICLICK_SHIFT;
+	/* Protect the mixer from broken values caused by xmp_set_tempo_factor. */
+	if (s->ticksize < 0 || s->ticksize > (XMP_MAX_FRAMESIZE / 2)) {
+		s->ticksize = XMP_MAX_FRAMESIZE / 2;
+	}
 
 	bytelen = s->ticksize * sizeof(int32);
 	if (~s->format & XMP_FORMAT_MONO) {
@@ -473,6 +524,7 @@ void libxmp_mixer_prepare(struct context_data *ctx)
 	}
 	memset(s->buf32, 0, bytelen);
 }
+
 /* Fill the output buffer calling one of the handlers. The buffer contains
  * sound for one tick (a PAL frame or 1/50s for standard vblank-timed mods)
  */
@@ -492,7 +544,7 @@ void libxmp_mixer_softmixer(struct context_data *ctx)
 	int prev_l, prev_r = 0;
 	int32 *buf_pos;
 	MIX_FP  mix_fn;
-	MIX_FP *mixerset;
+	const MIX_FP *mixerset;
 
 	switch (s->interp) {
 	case XMP_INTERP_NEAREST:
@@ -566,21 +618,24 @@ void libxmp_mixer_softmixer(struct context_data *ctx)
 		}
 
 		if (vi->pan == PAN_SURROUND) {
-			vol_r = vol * 0x80;
-			vol_l = -vol * 0x80;
+			vol_l = vol * 0x80;
+			vol_r = -vol * 0x80;
 		} else {
-			vol_r = vol * (0x80 - vi->pan);
-			vol_l = vol * (0x80 + vi->pan);
+			vol_l = vol * (0x80 - vi->pan);
+			vol_r = vol * (0x80 + vi->pan);
 		}
 
-		if (vi->smp < mod->smp) {
-			xxs = &mod->xxs[vi->smp];
-			xtra = &m->xtra[vi->smp];
-			c5spd = m->xtra[vi->smp].c5spd;
+		/* Sample is paused - skip channel unless a new sample is queued. */
+		if (vi->flags & SAMPLE_PAUSED) {
+			if ((~vi->flags & SAMPLE_QUEUED) || vi->queued.smp < 0) {
+				vi->flags &= ~SAMPLE_QUEUED;
+				continue;
+			}
+			hotswap_sample(ctx, vi, voc, vi->queued.smp);
+			get_current_sample(ctx, vi, &xxs, &xtra, &c5spd);
+			vi->pos = vi->start;
 		} else {
-			xxs = &ctx->smix.xxs[vi->smp - mod->smp];
-			xtra = NULL;
-			c5spd = m->c4rate;
+			get_current_sample(ctx, vi, &xxs, &xtra, &c5spd);
 		}
 
 		step = C4_PERIOD * c5spd / s->freq / vi->period;
@@ -592,7 +647,6 @@ void libxmp_mixer_softmixer(struct context_data *ctx)
 			continue;
 		}
 
-		adjust_voice_end(ctx, vi, xxs, xtra);
 		init_sample_wraparound(s, &loop_data, vi, xxs);
 
 		rampsize = s->ticksize >> ANTICLICK_SHIFT;
@@ -649,9 +703,11 @@ void libxmp_mixer_softmixer(struct context_data *ctx)
 				/* For Hipolito's anticlick routine */
 				if (samples > 0) {
 					if (~s->format & XMP_FORMAT_MONO) {
-						prev_r = buf_pos[mix_size - 2];
+						prev_l = buf_pos[mix_size - 2];
+						prev_r = buf_pos[mix_size - 1];
+					} else {
+						prev_l = buf_pos[mix_size - 1];
 					}
-					prev_l = buf_pos[mix_size - 1];
 				} else {
 					prev_r = prev_l = 0;
 				}
@@ -691,48 +747,69 @@ void libxmp_mixer_softmixer(struct context_data *ctx)
 					vi->old_vl += samples * delta_l;
 					vi->old_vr += samples * delta_r;
 
-
 					/* For Hipolito's anticlick routine */
 					if (~s->format & XMP_FORMAT_MONO) {
-						vi->sright = buf_pos[-2] - prev_r;
+						vi->sleft = buf_pos[-2] - prev_l;
+						vi->sright = buf_pos[-1] - prev_r;
+					} else {
+						vi->sleft = buf_pos[-1] - prev_l;
 					}
-					vi->sleft = buf_pos[-1] - prev_l;
 				}
 			}
 
 			vi->pos += step_dir * samples;
-
-			/* No more samples in this tick */
 			size -= samples;
-			if (size <= 0) {
-				if (has_active_loop(ctx, vi, xxs)) {
-					/* This isn't particularly important for
-					 * forward loops, but reverse loops need
-					 * to be corrected here to avoid their
-					 * negative positions getting clamped
-					 * in later ticks. */
-					if (((~vi->flags & VOICE_REVERSE) && vi->pos >= vi->end) ||
-					    ((vi->flags & VOICE_REVERSE) && vi->pos <= vi->start)) {
-						if (loop_reposition(ctx, vi, xxs, xtra)) {
-							reset_sample_wraparound(&loop_data);
-							init_sample_wraparound(s, &loop_data, vi, xxs);
-						}
-					}
-				}
-				continue;
-			}
 
-			/* First sample loop run */
-			if (!has_active_loop(ctx, vi, xxs) || split_noloop) {
-				do_anticlick(ctx, voc, buf_pos, size);
-				set_sample_end(ctx, voc, 1);
+			/* One-shot samples do not loop. */
+			if ((!has_active_loop(ctx, vi, xxs) || split_noloop) &&
+			    !(vi->flags & SAMPLE_QUEUED)) {
+				if (size > 0) {
+					do_anticlick(ctx, voc, buf_pos, size);
+					set_sample_end(ctx, voc, 1);
+					/* Next sample should ramp. */
+					vol_l = vol_r = 0;
+				}
 				size = 0;
 				continue;
 			}
 
-			if (loop_reposition(ctx, vi, xxs, xtra)) {
-				reset_sample_wraparound(&loop_data);
-				init_sample_wraparound(s, &loop_data, vi, xxs);
+			/* Loop before continuing to the next channel if the
+			 * tick is complete. This is particularly important
+			 * for reverse loops to avoid position clamping. */
+			if (size > 0 ||
+			    ((~vi->flags & VOICE_REVERSE) && vi->pos >= vi->end) ||
+			     ((vi->flags & VOICE_REVERSE) && vi->pos <= vi->start)) {
+				if (vi->flags & SAMPLE_QUEUED) {
+					/* Protracker sample swap */
+					do_anticlick(ctx, voc, buf_pos, size);
+					if (vi->queued.smp < 0 ||
+					    (!has_active_loop(ctx, vi, xxs) &&
+					     !(mod->xxs[vi->queued.smp].flg & XMP_SAMPLE_LOOP))) {
+						/* Invalid samples and one-shots that
+						 * are being replaced by one-shots
+						 * (OpenMPT PTStoppedSwap.mod) stop
+						 * the current sample. If the current
+						 * sample is looped, it needs to be paused.
+						 */
+						vi->flags &= ~SAMPLE_QUEUED;
+						vi->flags |= SAMPLE_PAUSED;
+						set_sample_end(ctx, voc, 1);
+						/* Next sample should ramp. */
+						vol_l = vol_r = 0;
+						size = 0;
+						continue;
+					}
+					reset_sample_wraparound(&loop_data);
+					hotswap_sample(ctx, vi, voc, vi->queued.smp);
+					get_current_sample(ctx, vi, &xxs, &xtra, &c5spd);
+					init_sample_wraparound(s, &loop_data, vi, xxs);
+					vi->pos = vi->start;
+					continue;
+				}
+				if (loop_reposition(ctx, vi, xxs, xtra)) {
+					reset_sample_wraparound(&loop_data);
+					init_sample_wraparound(s, &loop_data, vi, xxs);
+				}
 			}
 		}
 
@@ -771,6 +848,18 @@ void libxmp_mixer_voicepos(struct context_data *ctx, int voc, double pos, int ac
 	struct xmp_sample *xxs;
 	struct extra_sample_data *xtra;
 
+	/* Position changes e.g. retrigger make the new sample take effect
+	 * if queued (OpenMPT InstrSwapRetrigger.mod). */
+	if (vi->flags & SAMPLE_QUEUED) {
+		vi->flags &= ~SAMPLE_QUEUED;
+		if (vi->queued.smp < 0) {
+			vi->flags |= SAMPLE_PAUSED;
+		} else if (vi->smp != vi->queued.smp) {
+			hotswap_sample(ctx, vi, voc, vi->queued.smp);
+		}
+		vi->flags |= SAMPLE_LOOP;
+	}
+
 	if (vi->smp < m->mod.smp) {
 		xxs = &m->mod.xxs[vi->smp];
 		xtra = &m->xtra[vi->smp];
@@ -832,12 +921,12 @@ void libxmp_mixer_setpatch(struct context_data *ctx, int voc, int smp, int ac)
 	vi->smp = smp;
 	vi->vol = 0;
 	vi->pan = 0;
-	vi->flags &= ~(SAMPLE_LOOP | VOICE_REVERSE | VOICE_BIDIR);
+	vi->flags &= ~(SAMPLE_LOOP | SAMPLE_QUEUED | SAMPLE_PAUSED | VOICE_REVERSE | VOICE_BIDIR);
 
 	vi->fidx = 0;
 
 	if (~s->format & XMP_FORMAT_MONO) {
-		vi->fidx |= FLAG_STEREO;
+		vi->fidx |= FLAG_STEREOOUT;
 	}
 
 	set_sample_end(ctx, voc, 0);
@@ -856,10 +945,32 @@ void libxmp_mixer_setpatch(struct context_data *ctx, int voc, int smp, int ac)
 	if (xxs->flg & XMP_SAMPLE_16BIT) {
 		vi->fidx |= FLAG_16_BITS;
 	}
+	if (xxs->flg & XMP_SAMPLE_STEREO) {
+		vi->fidx |= FLAG_STEREO;
+	}
 
 	libxmp_mixer_voicepos(ctx, voc, 0, ac);
 }
 
+/**
+ * Replace the current playing sample when it reaches the end of its
+ * sample loop, a la Protracker 1/2. The new sample will begin playing
+ * at the start of its loop if it is looped, the start of the sample if
+ * it is a one-shot, and it will not play and instead pause the channel
+ * if both the original and the new sample are one-shots or if the new
+ * sample is empty/invalid/-1.
+ */
+void libxmp_mixer_queuepatch(struct context_data *ctx, int voc, int smp)
+{
+	struct player_data *p = &ctx->p;
+	struct mixer_voice *vi = &p->virt.voice_array[voc];
+
+	if (smp != vi->smp || (vi->flags & SAMPLE_PAUSED)) {
+		vi->queued.smp = smp;
+		vi->flags |= SAMPLE_QUEUED;
+	}
+}
+
 void libxmp_mixer_setnote(struct context_data *ctx, int voc, int note)
 {
 	struct player_data *p = &ctx->p;
@@ -989,11 +1100,11 @@ int libxmp_mixer_on(struct context_data *ctx, int rate, int format, int c4rate)
 {
 	struct mixer_data *s = &ctx->s;
 
-	s->buffer = (char *) calloc(2, XMP_MAX_FRAMESIZE);
+	s->buffer = (char *) calloc(XMP_MAX_FRAMESIZE, sizeof(int16));
 	if (s->buffer == NULL)
 		goto err;
 
-	s->buf32 = (int32 *) calloc(sizeof(int32), XMP_MAX_FRAMESIZE);
+	s->buf32 = (int32 *) calloc(XMP_MAX_FRAMESIZE, sizeof(int32));
 	if (s->buf32 == NULL)
 		goto err1;
 
