@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2023 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2025 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -37,6 +37,7 @@
  * Claudio's fix: implementing effect K
  */
 
+#include "common.h"
 #include "virtual.h"
 #include "period.h"
 #include "effects.h"
@@ -117,7 +118,9 @@ static int get_envelope(struct xmp_envelope *env, int x, int def)
 	return x2 == x1 ? y2 : ((y2 - y1) * (x - x1) / (x2 - x1)) + y1;
 }
 
-static int update_envelope_xm(struct xmp_envelope *env, int x, int release)
+#ifndef LIBXMP_CORE_PLAYER
+
+static int update_envelope_generic(struct xmp_envelope *env, int x, int release)
 {
 	int16 *data = env->data;
 	int has_loop, has_sus;
@@ -135,6 +138,9 @@ static int update_envelope_xm(struct xmp_envelope *env, int x, int release)
 	 * envelope loop end and the key is released, FT2 escapes the loop
 	 * while IT runs another iteration. (See EnvLoops.xm in the OpenMPT
 	 * test cases.)
+	 * TODO: this is a bit suspicious, has little relation to the above
+	 * description, and had to be removed from the XM handler because it
+	 * broke a module (fade_2_grey_visage.xm). Retesting is required.
 	 */
 	if (has_loop && has_sus && sus == lpe) {
 		if (!release)
@@ -142,8 +148,8 @@ static int update_envelope_xm(struct xmp_envelope *env, int x, int release)
 	}
 
 	/* If the envelope point is set to somewhere after the sustain point
-	 * or sustain loop, enable release to prevent the envelope point to
-	 * return to the sustain point or loop start. (See Filip Skutela's
+	 * or sustain loop, enable release to prevent the envelope point from
+	 * returning to the sustain point or loop start. (See Filip Skutela's
 	 * farewell_tear.xm.)
 	 */
 	if (has_loop && x > data[lpe] + 1) {
@@ -159,8 +165,71 @@ static int update_envelope_xm(struct xmp_envelope *env, int x, int release)
 		}
 	}
 
-	/* Envelope loops */
+	/* XM-like formats and players assume that an envelope position past the
+	 * end of the loop or sustain point should return to the loop/sustain point.
+	 * While there are some differences with sustain points, this general loop
+	 * behavior is used by DigiBooster Pro, Digitrakker, Imago Orpheus, and
+	 * Real Tracker 2.
+	 */
 	if (has_loop && x >= data[lpe]) {
+		/* FT2 and IT envelopes behave in a different way regarding
+		 * loops, sustain and release. When the sustain point is at the
+		 * end of the envelope loop end and the key is released, FT2
+		 * escapes the loop while IT runs another iteration.
+		 * (See OpenMPT EnvLoops.xm)
+		 */
+		if (!(release && has_sus && sus == lpe))
+			x = data[lps];
+	}
+
+	return x;
+}
+
+#endif
+
+static int update_envelope_xm(struct xmp_envelope *env, int x, int release)
+{
+	int16 *data = env->data;
+	int has_loop, has_sus;
+	int lpe, lps, sus;
+
+	has_loop = env->flg & XMP_ENVELOPE_LOOP;
+	has_sus = env->flg & XMP_ENVELOPE_SUS;
+
+	lps = env->lps << 1;
+	lpe = env->lpe << 1;
+	sus = env->sus << 1;
+
+	/* If the envelope point is set to somewhere after the sustain point
+	 * or sustain loop, enable release to prevent the envelope point from
+	 * returning to the sustain point or loop start. (See Filip Skutela's
+	 * farewell_tear.xm.)
+	 */
+	if (has_sus && x > data[sus] + 1) {
+		release = 1;
+	}
+
+	/* If enabled, stay at the sustain point */
+	if (has_sus && !release) {
+		if (x >= data[sus]) {
+			x = data[sus];
+		}
+	}
+
+	/* Envelope loops
+	 *
+	 * If the envelope point is set to somewhere after the sustain point
+	 * or sustain loop, the loop point is ignored to prevent the envelope
+	 * point from returning to the sustain point or loop start.
+	 * (See Filip Skutela's farewell_tear.xm or Ebony Owl Netsuke.xm.)
+	*/
+	if (has_loop && x == data[lpe]) {
+		/* FT2 and IT envelopes behave in a different way regarding
+		 * loops, sustain and release. When the sustain point is at the
+		 * end of the envelope loop end and the key is released, FT2
+		 * escapes the loop while IT runs another iteration.
+		 * (See OpenMPT EnvLoops.xm)
+		 */
 		if (!(release && has_sus && sus == lpe))
 			x = data[lps];
 	}
@@ -206,8 +275,10 @@ static int update_envelope_it(struct xmp_envelope *env, int x, int release, int
 
 #endif
 
-static int update_envelope(struct xmp_envelope *env, int x, int release, int key_off, int it_env)
+static int update_envelope(struct context_data *ctx, struct xmp_envelope *env, int x, int release, int key_off)
 {
+	struct module_data *m = &ctx->m;
+
 	if (x < 0xffff)	{	/* increment tick */
 		x++;
 	}
@@ -220,13 +291,18 @@ static int update_envelope(struct xmp_envelope *env, int x, int release, int key
 		return x;
 	}
 
+	(void) m; /* unused in xmp-lite with IT disabled */
+
+	return
 #ifndef LIBXMP_CORE_DISABLE_IT
-	return it_env ?
+		IS_PLAYER_MODE_IT() ?
 		update_envelope_it(env, x, release, key_off) :
-		update_envelope_xm(env, x, release);
-#else
-	return update_envelope_xm(env, x, release);
 #endif
+#ifndef LIBXMP_CORE_PLAYER
+		!HAS_QUIRK(QUIRK_FT2ENV) ?
+		update_envelope_generic(env, x, release) :
+#endif
+		update_envelope_xm(env, x, release);
 }
 
 
@@ -486,7 +562,12 @@ static void update_invloop(struct context_data *ctx, struct channel_data *xc)
 {
 	struct xmp_sample *xxs = libxmp_get_sample(ctx, xc->smp);
 	struct module_data *m = &ctx->m;
-	int lps, len = -1;
+	int lps = 0, len = -1;
+
+	/* If an instrument number is present, reset the position. */
+	if (ctx->p.frame == 0 && TEST(NEW_INS)) {
+		xc->invloop.pos = 0;
+	}
 
 	xc->invloop.count += invloop_table[xc->invloop.speed];
 
@@ -699,6 +780,19 @@ static int check_delay(struct context_data *ctx, struct xmp_event *e, int chn)
 		xc->delay = LSN(e->f2p) + 1;
 		goto do_delay;
 	}
+#ifndef LIBXMP_CORE_PLAYER
+	/* MED retrigger: reset retrigger so it doesn't continue during the delay. */
+	if (e->fxt == FX_MED_RETRIG && MSN(e->fxp)) {
+		RESET(RETRIG);
+		xc->delay = MSN(e->fxp) + 1;
+		goto do_delay;
+	}
+	if (e->f2t == FX_MED_RETRIG && MSN(e->f2p)) {
+		RESET(RETRIG);
+		xc->delay = MSN(e->fxp) + 1;
+		goto do_delay;
+	}
+#endif
 
 	return 0;
 
@@ -897,8 +991,8 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 	}
 
 	if (!TEST_PER(VENV_PAUSE)) {
-		xc->v_idx = update_envelope(&instrument->aei, xc->v_idx,
-			DOENV_RELEASE, TEST(KEY_OFF), IS_PLAYER_MODE_IT());
+		xc->v_idx = update_envelope(ctx, &instrument->aei, xc->v_idx,
+			DOENV_RELEASE, TEST(KEY_OFF));
 	}
 
 	vol_envelope = get_envelope(&instrument->aei, xc->v_idx, 64);
@@ -1036,8 +1130,8 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	instrument = libxmp_get_instrument(ctx, xc->ins);
 
 	if (!TEST_PER(FENV_PAUSE)) {
-		xc->f_idx = update_envelope(&instrument->fei, xc->f_idx,
-			DOENV_RELEASE, TEST(KEY_OFF), IS_PLAYER_MODE_IT());
+		xc->f_idx = update_envelope(ctx, &instrument->fei, xc->f_idx,
+			DOENV_RELEASE, TEST(KEY_OFF));
 	}
 	frq_envelope = get_envelope(&instrument->fei, xc->f_idx, 0);
 
@@ -1246,8 +1340,8 @@ static void process_pan(struct context_data *ctx, int chn, int act)
 	instrument = libxmp_get_instrument(ctx, xc->ins);
 
 	if (!TEST_PER(PENV_PAUSE)) {
-		xc->p_idx = update_envelope(&instrument->pei, xc->p_idx,
-			DOENV_RELEASE, TEST(KEY_OFF), IS_PLAYER_MODE_IT());
+		xc->p_idx = update_envelope(ctx, &instrument->pei, xc->p_idx,
+			DOENV_RELEASE, TEST(KEY_OFF));
 	}
 	pan_envelope = get_envelope(&instrument->pei, xc->p_idx, 32);
 
@@ -1551,7 +1645,7 @@ static void play_channel(struct context_data *ctx, int chn)
 			xc->volume += rval[xc->retrig.type].s;
 			xc->volume *= rval[xc->retrig.type].m;
 			xc->volume /= rval[xc->retrig.type].d;
-			xc->retrig.count = LSN(xc->retrig.val);
+			xc->retrig.count = xc->retrig.val;
 
 			if (xc->retrig.limit > 0) {
 				/* Limit the number of retriggers. */
@@ -1624,6 +1718,7 @@ static void next_order(struct context_data *ctx)
 	struct xmp_module *mod = &m->mod;
 	int reset_gvol = 0;
 	int mark;
+	int i;
 
 	do {
 		p->ord++;
@@ -1666,6 +1761,17 @@ static void next_order(struct context_data *ctx)
 	p->pos = p->ord;
 	p->frame = 0;
 
+	/* Scream Tracker 3, Imago Orpheus: position change resets loop vars.
+	 * For some reason the pattern jump effect does not do this in IMF. */
+	if (HAS_FLOW_MODE(FLOW_LOOP_PATTERN_RESET)) {
+		f->loop_start = -1;
+		f->loop_count = 0;
+		for (i = 0; i < mod->chn; i++) {
+			f->loop[i].start = 0;
+			f->loop[i].count = 0;
+		}
+	}
+
 #ifndef LIBXMP_CORE_PLAYER
 	f->jump_in_pat = -1;
 
@@ -1686,6 +1792,7 @@ static void next_row(struct context_data *ctx)
 
 	p->frame = 0;
 	f->delay = 0;
+	f->loop_param = -1;
 
 	if (f->pbreak) {
 		f->pbreak = 0;
@@ -1704,9 +1811,9 @@ static void next_row(struct context_data *ctx)
 			f->rowdelay--;
 		}
 
-		if (f->loop_chn) {
-			p->row = f->loop[f->loop_chn - 1].start;
-			f->loop_chn = 0;
+		if (f->loop_dest >= 0) {
+			p->row = f->loop_dest;
+			f->loop_dest = -1;
 		}
 
 		/* check end of pattern */
@@ -1763,7 +1870,11 @@ void libxmp_reset_flow(struct context_data *ctx)
 	f->jumpline = 0;
 	f->jump = -1;
 	f->pbreak = 0;
-	f->loop_chn = 0;
+	f->loop_dest = -1;
+	f->loop_param = -1;
+	f->loop_start = -1;
+	f->loop_count = 0;
+	f->loop_active_num = 0;
 	f->delay = 0;
 	f->rowdelay = 0;
 	f->rowdelay_set = 0;
