@@ -56,20 +56,30 @@ struct retrig_control {
 #define FINE_VOLS_2	(1 << 25)
 #define KEY_OFF		(1 << 26)	/* for IT release on envloop end */
 #define TREMOR		(1 << 27)	/* for XM tremor */
+#define MIDI_MACRO	(1 << 28)	/* IT midi macro */
 
 #define NOTE_FADEOUT	(1 << 0)
-#define NOTE_RELEASE	(1 << 1)
+#define NOTE_ENV_RELEASE (1 << 1)	/* envelope sustain loop release */
 #define NOTE_END	(1 << 2)
 #define NOTE_CUT	(1 << 3)
 #define NOTE_ENV_END	(1 << 4)
 #define NOTE_SAMPLE_END	(1 << 5)
 #define NOTE_SET	(1 << 6)	/* for IT portamento after keyoff */
-#define NOTE_SUSEXIT	(1 << 7)	/* for delayed note release */
+#define NOTE_SUSEXIT	(1 << 7)	/* for delayed envelope release */
 #define NOTE_KEY_CUT	(1 << 8)	/* note cut with XMP_KEY_CUT event */
 #define NOTE_GLISSANDO	(1 << 9)
+#define NOTE_SAMPLE_RELEASE (1 << 10)	/* sample sustain loop release */
 
+/* Most of the time, these should be set/reset together. */
+#define NOTE_RELEASE	(NOTE_ENV_RELEASE | NOTE_SAMPLE_RELEASE)
+
+/* Note: checking the data pointer for samples should be good enough to filter
+ * broken samples, since libxmp_load_sample will always allocate it for valid
+ * samples of >0 length and bound the loop values for these samples. */
 #define IS_VALID_INSTRUMENT(x) ((uint32)(x) < mod->ins && mod->xxi[(x)].nsm > 0)
 #define IS_VALID_INSTRUMENT_OR_SFX(x) (((uint32)(x) < mod->ins && mod->xxi[(x)].nsm > 0) || (smix->ins > 0 && (uint32)(x) < mod->ins + smix->ins))
+#define IS_VALID_SAMPLE(x) ((uint32)(x) < mod->smp && mod->xxs[(x)].data != NULL)
+#define IS_VALID_NOTE(x) ((uint32)(x) < XMP_MAX_KEYS)
 
 struct instrument_vibrato {
 	int phase;
@@ -86,7 +96,7 @@ struct channel_data {
 	double per_adj;		/* MED period/pitch adjustment factor hack */
 	int finetune;		/* Guess what */
 	int ins;		/* Instrument number */
-	int old_ins;		/* Last instruemnt */
+	int old_ins;		/* Last instrument */
 	int smp;		/* Sample number */
 	int mastervol;		/* Master vol -- for IT track vol effect */
 	int delay;		/* Note delay in frames */
@@ -147,6 +157,7 @@ struct channel_data {
 		int val;	/* Retrig value */
 		int count;	/* Retrig counter */
 		int type;	/* Retrig type */
+		int limit;	/* Number of retrigs */
 	} retrig;
 
 	struct {
@@ -163,6 +174,9 @@ struct channel_data {
 #ifndef LIBXMP_CORE_DISABLE_IT
 		int fslide2;
 		int memory2;	/* Volume slide effect memory */
+#endif
+#ifndef LIBXMP_CORE_PLAYER
+		int target;	/* Target for persistent volslide */
 #endif
 	} vol;
 
@@ -187,18 +201,22 @@ struct channel_data {
 		int slide;	/* Frequency slide value */
 		double fslide;	/* Fine frequency slide value */
 		int memory;	/* Portamento effect memory */
+		int down_memory;/* Portamento down effect memory (XM) */
 	} freq;
 
 	struct {
 		double target;	/* Target period for tone portamento */
-		int dir;	/* Tone portamento up/down directionh */
+		int dir;	/* Tone portamento up/down direction */
 		int slide;	/* Delta for tone portamento */
 		int memory;	/* Tone portamento effect memory */
+		int note_memory;/* Tone portamento note memory (ULT) */
 	} porta;
 
 	struct {
 		int up_memory;	/* FT2 has separate memories for these */
 		int down_memory;/* cases (see Porta-LinkMem.xm) */
+		int xf_up_memory;
+		int xf_down_memory;
 	} fine_porta;
 
 	struct {
@@ -207,7 +225,7 @@ struct channel_data {
 		int fslide;	/* Pan fine slide value */
 		int memory;	/* Pan slide effect memory */
 		int surround;	/* Surround channel flag */
-	} pan;	
+	} pan;
 
 	struct {
 		int speed;
@@ -224,8 +242,17 @@ struct channel_data {
 		int cutoff;	/* IT filter cutoff frequency */
 		int resonance;	/* IT filter resonance */
 		int envelope;	/* IT filter envelope */
+		int can_disable;/* IT hack: allow disabling for cutoff 127 */
 	} filter;
 
+	struct {
+		float val;	/* Current macro effect (use float for slides) */
+		float target;	/* Current macro target (smooth macro) */
+		float slide;	/* Current macro slide (smooth macro) */
+		int active;	/* Current active parameterized macro */
+		int finalvol;	/* Previous tick calculated volume (0-0x400) */
+		int notepan;	/* Previous tick note panning (0x80 center) */
+	} macro;
 #endif
 
 #ifndef LIBXMP_CORE_PLAYER
@@ -252,6 +279,8 @@ struct channel_data {
 
 void	libxmp_process_fx	(struct context_data *, struct channel_data *,
 				 int, struct xmp_event *, int);
+void	libxmp_process_pattern_loop	(struct context_data *,
+					 struct flow_control *f, int, int, int);
 void	libxmp_filter_setup	(int, int, int, int*, int*, int *);
 int	libxmp_read_event	(struct context_data *, struct xmp_event *, int);
 
